<?php

namespace EventPortal\Domain\Delegate\Models;

use EventPortal\Concerns\AddToCasts;
use EventPortal\Concerns\AddToFillable;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Model;
use EventPortal\Domain\Table\Concerns\HasTable;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;

class Delegate extends Model
{
    use HasTable, AddToFillable, AddToCasts;

    const KEY = 'delegate';

    /** @var array<string> */
    public $with = [
        'user',
        'events',
    ];

    /** @var array<string> */
    protected $fillable = [
        'user_id',
    ];

    public function events(): BelongsToMany
    {
        return $this
            ->belongsToMany(Event::class)
            ->withPivot(['status'])
            ->withTimestamps();
    }

    public function user(): BelongsTo
    {
        return $this->belongsTo(config('auth.providers.users.model'), 'user_id');
    }

    public function getNameAttribute(): string
    {
        return $this->user->first_name . ' ' . $this->user->last_name;
    }
}
