<?php

namespace EventPortal\Domain\Table;

use EventPortal\Domain\Component\Value;
use Illuminate\Database\Eloquent\Model;

class TableRow
{
    /** @var GroupedActions */
    public $action = [];

    public function __construct(Model $model, ?callable $action, ?array $context = [])
    {
        $this->setFields($model);

        // phpcs:disable SlevomatCodingStandard.ControlStructures.EarlyExit.EarlyExitNotUsed
        if ($action !== null) {
            $this->setActions($model, $action, $context);
        }
        // phpcs:enable
    }

    private function setFields(Model $model): void
    {
        collect($model->getAttributes())
            // phpcs:disable Generic.CodeAnalysis.UnusedFunctionParameter
            ->each(function ($value, $key) use ($model): void {
            // phpcs:enable
                $this->$key = Value::get($model->$key);
            });
    }

    private function setActions(Model $model, callable $action, ?array $context = []): void
    {
        $models = [$model::KEY => $model];

        if ($context !== null) {
            $models = array_merge($context, [$model::KEY => $model]);
        }

        $this->action = app()->call($action, $models);
    }
}
