<?php

namespace EventPortal\Tests\Unit\Table;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Permission\Models\Permission;
use EventPortal\Domain\Table\GroupedActions;
use EventPortal\Domain\User\Models\User;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Permission\Exceptions\PermissionDoesNotExist;
use Spatie\Snapshots\MatchesSnapshots;

class GroupedActionsTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots;

    /** @test */
    public function can_add_actions()
    {
        $groupActions = GroupedActions::make('Manage event')
            ->actions([
                Button::make('View'),
                Button::make('Delete'),
            ]);

        $this->assertMatchesJsonSnapshot(json_encode($groupActions->actions));
    }

    /** @test */
    public function adds_when_user_has_permissions()
    {
        $user = factory(User::class)->create();
        auth()->login($user);
        factory(Permission::class)->create([
            'name' => 'can add button',
            'guard_name' => 'web',
        ]);
        $user->givePermissionTo('can add button');

        $groupActions = GroupedActions::make('Manage event')
            ->addIfUserHasPermission('can add button', static function () {
                return Button::make('View');
            });

        $this->assertMatchesJsonSnapshot(json_encode($groupActions->actions));
    }

    /** @test */
    public function does_not_add_when_user_does_not_have_permission()
    {
        $user = factory(User::class)->create();
        auth()->login($user);

        try {
            GroupedActions::make('Manage event')
                ->addIfUserHasPermission('can add button', static function () {
                    return Button::make('View');
                });
        } catch (PermissionDoesNotExist $exception) {
            $this->assertEquals('There is no permission named `can add button` for guard `web`.', $exception->getMessage());
            return;
        }

        $this->fail('Added button when we do not have permission to');
    }
}
